/**
 * @file buss.cpp
 * @author Andreas Blakli
 */

#include "buss.h"

using namespace std;

/**
 * Constructor for Buss, lager ett Buss objekt.
 *
 */
Buss::Buss() : Rute() {
}

/**
 * Destructor for Buss objekt.
 *
 */
Buss::~Buss() {
}

/**
 * Leser inn data om bussen fra CLI.
 *
 * @see lesData3 lesInt().
 * @see lesData3 lesChar().
 */
void Buss::lesData() {
    int valg;
    antallSitteplasser = lesInt("Antall sitteplasser", MIN_ANTALL_SITTEPLASSER,
                                MAX_ANTALL_SITTEPLASSER);
    antallStaplasser = lesInt("Antall ståplasser", MIN_ANTALL_STAPLASSER,
                              MAX_ANTALL_STAPLASSER);
    do {
        valg = lesChar("Leddbuss (J/n)");
    } while (valg != 'J' && valg != 'N');

    if (valg == 'J')
        leddbuss = true;
    else
        leddbuss = false;
}

/**
 * Skriver ut data om bussen til terminal.
 *
 */
void Buss::skrivData() {
    cout << "\n\t..::Buss info::.." << '\n';
    cout << "\tAntall sitteplasser: " << antallSitteplasser << '\n'
         << "\tAntall ståplasser:" << antallStaplasser << '\n'
         << "\tEr leddbuss: ";
    if (leddbuss)
        cout << "Ja";
    else
        cout << "Nei";
    cout << '\n';
}

/**
 * Skriver ut at typen er buss til terminal.
 *
 */
void Buss::skrivType() {
    std::cout << "Buss";
}

/**
 * Skriver ut data om bussen.
 *
 * @see skrivData().
 * @see Rute::SkrivDataForlengs().
 */
void Buss::skrivDataForlengs() {
    skrivData();
    Rute::skrivDataForlengs();
    cout << "\n\n";
}

/**
 * Skriver ut data om bussen.
 *
 * @see skrivData().
 * @see Rute::SkrivDataBaklengs().
 */
void Buss::skrivDataBaklengs() {
    skrivData();
    Rute::skrivDataBaklengs();
    cout << "\n\n";
}

/**
 * Leser inn bussdata fra fil.
 *
 * @param inn - Inn data fil-stream.
 *
 * @see HjelpeFunksjoner::splittStreng().
 * @see Rute::lesFraFil().
 */
void Buss::lesFraFil(ifstream& inn) {
    int i = 0;
    string heleLinjen = "";
    getline(inn, heleLinjen);
    vector<string> linjenSplittet = hjelp.splittStreng('*', heleLinjen);
    for (const auto& val : linjenSplittet) {
        switch (i) {
            case 0:
                antallSitteplasser = stoi(val);
                break;
            case 1:
                antallStaplasser = stoi(val);
                break;
            case 2:
                leddbuss = stoi(val);
                break;
        }
        i++;
    }
    Rute::lesFraFil(inn);
}

/**
 * Skriver buss til fil.
 *
 * @param ut - Ut data fil-stream.
 *
 * @see Rute::skrivTilFil().
 */
void Buss::skrivTilFil(ofstream& ut) {
    ut << "Buss" << '\n';
    ut << antallSitteplasser << '*'
       << antallStaplasser << '*'
       << leddbuss << '*'
       << '\n';
    Rute::skrivTilFil(ut);
}
