/**
 * @file stoppesteder.cpp
 * @author Andreas Blakli
 */

#include "stoppesteder.h"

using namespace std;

/**
 * Constructor for Stoppested, lager ett nytt Stoppestd object.
 *
 */
Stoppesteder::Stoppesteder() {
    lesFraFil();
}

/**
 * Destructor for Stoppested objekt.
 *
 */
Stoppesteder::~Stoppesteder() {
    skrivTilFil();
    ryddOppMinne();
}

/**
 * Skriver ut alle stoppesteder til terminal.
 *
 * @see Stoppested::hentNavn().
 */
void Stoppesteder::skrivData() {
    int i = 1;
    for (const auto& val : alleStoppesteder) {
        cout << "\n\t" << i << " - " << val->hentNavn();
        i++;
    }
}

/**
 * Skriver ut data om ett stoppested basert på bruker input
 * med info om nabostoppene e.g. navn og tid.
 *
 * @see HjelpeFunksjoner::rensStreng().
 * @see hentEksisterende().
 * @see Stoppested::skrivDataMedNaboer().
 */
void Stoppesteder::skrivSpesifikkData() {
    cout << "\nSkriv inn navnet på stoppestedet: ";
    string inn;
    getline(cin, inn);
    inn = hjelp.rensStreng(inn);
    Stoppested* stopp = hentEksisterende(inn);
    if (stopp != nullptr) {
        stopp->skrivDataMedNaboer(alleStoppesteder);
    } else
        cout << "\nIkke entydig eller finnes ikke!" << '\n';
}

/**
 * Legger til ett nytt stoppested hvis det ikke eksisterer.
 *
 * @param navn - Navnet på stoppestedet.
 * @param leggTilTid - Skal tid legges til ja/nei.
 *
 * @see hentEksisterende().
 */
void Stoppesteder::ny(const string& navn, const bool& leggTilTid) {
    if (hentEksisterende(navn) != nullptr)
        cout << '\n'
             << "Error: "
             << navn
             << " eksisterer allerede!"
             << '\n';
    else {
        alleStoppesteder.push_back(new Stoppested(navn, leggTilTid));
    }
}

/**
 * Returnerer pekeren til stoppestedet hvis det eksiterer.
 *
 * @param navn - Navnet på stoppestedet.
 * @return Stoppested* - Peker til stoppested, nullptr hvis det ikke finnes.
 *
 * @see HjelpeFunksjoner::gjorOmTilStorBokstav().
 * @see Stoppested::hentNavn().
 */
Stoppested* Stoppesteder::hentEksisterende(const string& navn) {
    bool lik;
    int antallLike = 0;
    string tmp;
    Stoppested* tmpStoppested = nullptr;
    for (const auto& val : alleStoppesteder) {
        if (hjelp.gjorOmTilStorBokstav(val->hentNavn()) ==
            hjelp.gjorOmTilStorBokstav(navn))
            return val;
        lik = true;
        tmp = "";
        tmp = val->hentNavn();
        for (int i = 0; i < navn.size(); i++) {
            if (i < tmp.size()) {
                if (toupper(navn[i]) != toupper(tmp[i])) lik = false;
            } else
                lik = false;
        }
        if (lik) {
            tmpStoppested = val;
            antallLike++;
        }
    }
    if (antallLike == 1) return tmpStoppested;
    return nullptr;
}

/**
 * Returnerer pekeren til stoppestedet hvis det eksiterer.
 *
 * @param index - Indeksen til stoppestedet i vectoren alleStoppesteder.
 * @return Stoppested* - Peker til stoppested, nullptr hvis det ikke finnes.
 */
Stoppested* Stoppesteder::hentEksisterende(const int& index) {
    if (alleStoppesteder.size() >= index) return alleStoppesteder[index];
    return nullptr;
}

/**
 * Skriver ut meny valgene til terminal.
 *
 */
void Stoppesteder::skrivMeny() {
    cout << "\nN - Nytt stoppested" << '\n'
         << "A - Skriv alle stoppesteder" << '\n'
         << "E - Skriv alt om ett stoppested" << '\n'
         << "Q - Tilbake";
}

/**
 * Returnerer indeksen til stoppestedet fra alleStoppesteder vectoren.
 *
 * @param stoppested - Peker til stoppestedet.
 * @return int - Indeksen til stoppestedet, -1 hvis det ikke finnes.
 */
int Stoppesteder::hentIndex(Stoppested* stoppested) {
    int i = 0;
    for (const auto& val : alleStoppesteder) {
        if (val == stoppested) return i;
        i++;
    }
    return -1;
}

/**
 * Meny, styrer hvilke funksjoner som skal brukes
 * basert på bruker valget i parameteren.
 *
 * @param valg - Bruker valget.
 *
 * @see HjelpeFunksjoner::rensStreng().
 * @see skrivData().
 * @see skrivSpesifikkData().
 */
void Stoppesteder::meny(const char& valg) {
    string navn;
    switch (valg) {
        case 'N': {
            cout << "\nSkriv inn navn på stoppested:";
            getline(cin, navn);
            navn = hjelp.rensStreng(navn);
            if (navn != "") {
                ny(navn, false);
            }
            break;
        }
        case 'A':
            skrivData();
            break;
        case 'E':
            skrivSpesifikkData();
            break;
        case 'Q':
            break;
        default:
            cout << "\nUgyldig valg!" << '\n';
            break;
    }
}

/**
 * Sletter de lagrede stoppested pekerne fra alleStoppesteder
 * vectoren.
 *
 */
void Stoppesteder::ryddOppMinne() {
    for (int i = 0; i < alleStoppesteder.size(); i++) {
        delete alleStoppesteder[i];
    }
    alleStoppesteder.clear();
}

/**
 * Leser stoppesteder fra fil.
 *
 * @see Stoppested::lesFraFil().
 */
void Stoppesteder::lesFraFil() {
    int antallStoppesteder, i;
    antallStoppesteder = i = 0;
    ifstream fil(STI_STOPPESTEDER);
    if (fil) {
        fil >> antallStoppesteder;
        fil.ignore();
        while (!fil.eof() && i < antallStoppesteder) {
            alleStoppesteder.push_back(new Stoppested);
            alleStoppesteder.back()->lesFraFil(fil);
            i++;
        }
        fil.close();
    } else
        cout << "\nError: " << STI_STOPPESTEDER << " ble ikke funnet!"
             << '\n';
}

/**
 * Skriver alle stoppestedene til fil.
 *
 * @see Stoppested::skrivTilFil().
 */
void Stoppesteder::skrivTilFil() {
    ofstream fil(STI_STOPPESTEDER);
    fil << alleStoppesteder.size() << '\n';
    for (const auto& val : alleStoppesteder) {
        val->skrivTilFil(fil);
    }
}